@@ -306,7 +306,9 @@ SPI_freeDesktopList (Accessible **desktop_list)
  * @listener:  a pointer to the #AccessibleKeystrokeListener for which
  *             keystroke events are requested.
  * @keys:      a pointer to the #AccessibleKeySet indicating which
- *             keystroke events are requested, or #CSPI_KEYSET_ALL_KEYS.
+ *             keystroke events are requested, or #CSPI_KEYSET_ALL_KEYS
+ *             to indicate that all keycodes and keyvals for the specified
+ *             modifier set are to be included.
  * @modmask:   an #AccessibleKeyMaskType mask indicating which
  *             key event modifiers must be set in combination with @keys,
  *             events will only be reported for key events for which all
@@ -332,13 +334,13 @@ SPI_registerAccessibleKeystrokeListener (AccessibleKeystrokeListener  *listener,
 					 AccessibleKeyEventMask        eventmask,
 					 AccessibleKeyListenerSyncType sync_type)
 {
-  gint                                i, mask;
+  gint                                i;
   Accessibility_KeySet                key_set;
   Accessibility_KeyEventTypeSeq       key_events;
   Accessibility_ControllerEventMask   controller_event_mask;
   Accessibility_DeviceEventController device_event_controller;
   Accessibility_EventListenerMode     listener_mode;
-  Accessibility_KeyEventType          key_event_types [2];
+  Accessibility_EventType             key_event_types [2];
   SPIBoolean                          retval = FALSE;
 
   if (!listener)
@@ -381,11 +383,11 @@ SPI_registerAccessibleKeystrokeListener (AccessibleKeystrokeListener  *listener,
   key_events._buffer = key_event_types;
   if (eventmask & SPI_KEY_PRESSED)
     {
-      key_events._buffer[i++] = Accessibility_KEY_PRESSED;
+      key_events._buffer[i++] = Accessibility_KEY_PRESSED_EVENT;
     }
   if (eventmask & SPI_KEY_RELEASED)
     {
-      key_events._buffer[i++] = Accessibility_KEY_RELEASED;
+      key_events._buffer[i++] = Accessibility_KEY_RELEASED_EVENT;
     }
   key_events._length = i;
   
@@ -474,6 +476,7 @@ SPI_deregisterAccessibleKeystrokeListener (AccessibleKeystrokeListener *listener
  *             the events.
  * @eventmask: an #AccessibleDeviceEventMask mask indicating which
  *             types of key events are requested (#SPI_KEY_PRESSED, etc.).
+ * @filter: Unused parameter.
  *             
  * Register a listener for device events, for instance button events.
  *
@@ -488,7 +491,7 @@ SPI_registerDeviceEventListener (AccessibleDeviceListener  *listener,
   SPIBoolean                          retval = FALSE;
   Accessibility_EventTypeSeq          event_types;
   Accessibility_EventType             event_type_buffer[2];
-  gint                                i, mask;
+  gint                                i;
 
   if (!listener)
     {
@@ -533,6 +536,7 @@ SPI_registerDeviceEventListener (AccessibleDeviceListener  *listener,
  * SPI_deregisterDeviceEventListener:
  * @listener: a pointer to the #AccessibleDeviceListener for which
  *            device events are requested.
+ * @filter: Unused parameter.
  *
  * Removes a device event listener from the registry's listener queue,
  *            ceasing notification of events of the specified type.
@@ -543,7 +547,6 @@ SPIBoolean
 SPI_deregisterDeviceEventListener (AccessibleDeviceListener *listener,
 				   void                     *filter)
 {
-  Accessibility_ControllerEventMask   controller_event_mask;
   Accessibility_DeviceEventController device_event_controller;
   Accessibility_EventTypeSeq       event_types;
   Accessibility_EventType          event_type_buff[2];
